@@ -1,4 +1,4 @@
-// Copyright (C) 2019-2020 Joel Rosdahl and other contributors
+// Copyright (C) 2019-2021 Joel Rosdahl and other contributors
 //
 // See doc/AUTHORS.adoc for a complete list of contributors.
 //
@@ -27,11 +27,22 @@
 #include "File.hpp"
 #include "Logging.hpp"
 #include "Stat.hpp"
-#include "Statistic.hpp"
 #include "Util.hpp"
-#include "exceptions.hpp"
 #include "fmtmacros.hpp"
 
+#include <core/Statistic.hpp>
+#include <core/exceptions.hpp>
+#include <core/wincompat.hpp>
+#include <util/path.hpp>
+
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
 #include <algorithm>
 
 // Result data format
@@ -182,7 +193,7 @@ gcno_file_in_mangled_form(const Context& ctx)
 {
   const auto& output_obj = ctx.args_info.output_obj;
   const std::string abs_output_obj =
-    Util::is_absolute_path(output_obj)
+    util::is_absolute_path(output_obj)
       ? output_obj
       : FMT("{}/{}", ctx.apparent_cwd, output_obj);
   std::string hashified_obj = abs_output_obj;
@@ -196,6 +207,14 @@ gcno_file_in_unmangled_form(const Context& ctx)
   return Util::change_extension(ctx.args_info.output_obj, ".gcno");
 }
 
+FileSizeAndCountDiff&
+FileSizeAndCountDiff::operator+=(const FileSizeAndCountDiff& other)
+{
+  size_kibibyte += other.size_kibibyte;
+  count += other.count;
+  return *this;
+}
+
 Result::Reader::Reader(const std::string& result_path)
   : m_result_path(result_path)
 {
@@ -212,7 +231,7 @@ Result::Reader::read(Consumer& consumer)
     } else {
       return "No such result file";
     }
-  } catch (const Error& e) {
+  } catch (const core::Error& e) {
     return e.what();
   }
 }
@@ -220,13 +239,20 @@ Result::Reader::read(Consumer& consumer)
 bool
 Reader::read_result(Consumer& consumer)
 {
-  File file(m_result_path, "rb");
-  if (!file) {
-    // Cache miss.
-    return false;
+  FILE* file_stream;
+  File file;
+  if (m_result_path == "-") {
+    file_stream = stdin;
+  } else {
+    file = File(m_result_path, "rb");
+    if (!file) {
+      // Cache miss.
+      return false;
+    }
+    file_stream = file.get();
   }
 
-  CacheEntryReader cache_entry_reader(file.get(), k_magic, k_version);
+  CacheEntryReader cache_entry_reader(file_stream, k_magic, k_version);
 
   consumer.on_header(cache_entry_reader);
 
@@ -239,7 +265,7 @@ Reader::read_result(Consumer& consumer)
   }
 
   if (i != n_entries) {
-    throw Error("Too few entries (read {}, expected {})", i, n_entries);
+    throw core::Error("Too few entries (read {}, expected {})", i, n_entries);
   }
 
   cache_entry_reader.finalize();
@@ -260,7 +286,7 @@ Reader::read_entry(CacheEntryReader& cache_entry_reader,
     break;
 
   default:
-    throw Error("Unknown entry type: {}", marker);
+    throw core::Error("Unknown entry type: {}", marker);
   }
 
   UnderlyingFileTypeInt type;
@@ -273,7 +299,7 @@ Reader::read_entry(CacheEntryReader& cache_entry_reader,
   if (marker == k_embedded_file_marker) {
     consumer.on_entry_start(entry_number, file_type, file_len, nullopt);
 
-    uint8_t buf[READ_BUFFER_SIZE];
+    uint8_t buf[CCACHE_READ_BUFFER_SIZE];
     size_t remain = file_len;
     while (remain > 0) {
       size_t n = std::min(remain, sizeof(buf));
@@ -287,10 +313,11 @@ Reader::read_entry(CacheEntryReader& cache_entry_reader,
     auto raw_path = get_raw_file_path(m_result_path, entry_number);
     auto st = Stat::stat(raw_path, Stat::OnError::throw_error);
     if (st.size() != file_len) {
-      throw Error("Bad file size of {} (actual {} bytes, expected {} bytes)",
-                  raw_path,
-                  st.size(),
-                  file_len);
+      throw core::Error(
+        "Bad file size of {} (actual {} bytes, expected {} bytes)",
+        raw_path,
+        st.size(),
+        file_len);
     }
 
     consumer.on_entry_start(entry_number, file_type, file_len, raw_path);
@@ -311,20 +338,20 @@ Writer::write(FileType file_type, const std::string& file_path)
   m_entries_to_write.emplace_back(file_type, file_path);
 }
 
-optional<std::string>
+nonstd::expected<FileSizeAndCountDiff, std::string>
 Writer::finalize()
 {
   try {
-    do_finalize();
-    return nullopt;
-  } catch (const Error& e) {
-    return e.what();
+    return do_finalize();
+  } catch (const core::Error& e) {
+    return nonstd::make_unexpected(e.what());
   }
 }
 
-void
+FileSizeAndCountDiff
 Writer::do_finalize()
 {
+  FileSizeAndCountDiff file_size_and_count_diff{0, 0};
   uint64_t payload_size = 0;
   payload_size += 1; // n_entries
   for (const auto& pair : m_entries_to_write) {
@@ -341,8 +368,8 @@ Writer::do_finalize()
   CacheEntryWriter writer(atomic_result_file.stream(),
                           k_magic,
                           k_version,
-                          Compression::type_from_config(m_ctx.config),
-                          Compression::level_from_config(m_ctx.config),
+                          compression::type_from_config(m_ctx.config),
+                          compression::level_from_config(m_ctx.config),
                           payload_size);
 
   writer.write<uint8_t>(m_entries_to_write.size());
@@ -351,7 +378,7 @@ Writer::do_finalize()
   for (const auto& pair : m_entries_to_write) {
     const auto file_type = pair.first;
     const auto& path = pair.second;
-    LOG("Storing result {}", path);
+    LOG("Storing result file {}", path);
 
     const bool store_raw = should_store_raw_file(m_ctx.config, file_type);
     uint64_t file_size = Stat::stat(path, Stat::OnError::throw_error).size();
@@ -369,7 +396,7 @@ Writer::do_finalize()
     writer.write(file_size);
 
     if (store_raw) {
-      write_raw_file_entry(path, entry_number);
+      file_size_and_count_diff += write_raw_file_entry(path, entry_number);
     } else {
       write_embedded_file_entry(writer, path, file_size);
     }
@@ -379,6 +406,8 @@ Writer::do_finalize()
 
   writer.finalize();
   atomic_result_file.commit();
+
+  return file_size_and_count_diff;
 }
 
 void
@@ -388,29 +417,29 @@ Result::Writer::write_embedded_file_entry(CacheEntryWriter& writer,
 {
   Fd file(open(path.c_str(), O_RDONLY | O_BINARY));
   if (!file) {
-    throw Error("Failed to open {} for reading", path);
+    throw core::Error("Failed to open {} for reading", path);
   }
 
   uint64_t remain = file_size;
   while (remain > 0) {
-    uint8_t buf[READ_BUFFER_SIZE];
+    uint8_t buf[CCACHE_READ_BUFFER_SIZE];
     size_t n = std::min(remain, static_cast<uint64_t>(sizeof(buf)));
-    ssize_t bytes_read = read(*file, buf, n);
+    auto bytes_read = read(*file, buf, n);
     if (bytes_read == -1) {
       if (errno == EINTR) {
         continue;
       }
-      throw Error("Error reading from {}: {}", path, strerror(errno));
+      throw core::Error("Error reading from {}: {}", path, strerror(errno));
     }
     if (bytes_read == 0) {
-      throw Error("Error reading from {}: end of file", path);
+      throw core::Error("Error reading from {}: end of file", path);
     }
     writer.write(buf, bytes_read);
     remain -= bytes_read;
   }
 }
 
-void
+FileSizeAndCountDiff
 Result::Writer::write_raw_file_entry(const std::string& path,
                                      uint32_t entry_number)
 {
@@ -418,16 +447,15 @@ Result::Writer::write_raw_file_entry(const std::string& path,
   const auto old_stat = Stat::stat(raw_file);
   try {
     Util::clone_hard_link_or_copy_file(m_ctx, path, raw_file, true);
-  } catch (Error& e) {
-    throw Error(
+  } catch (core::Error& e) {
+    throw core::Error(
       "Failed to store {} as raw file {}: {}", path, raw_file, e.what());
   }
   const auto new_stat = Stat::stat(raw_file);
-  m_ctx.counter_updates.increment(
-    Statistic::cache_size_kibibyte,
-    Util::size_change_kibibyte(old_stat, new_stat));
-  m_ctx.counter_updates.increment(Statistic::files_in_cache,
-                                  (new_stat ? 1 : 0) - (old_stat ? 1 : 0));
+  return {
+    Util::size_change_kibibyte(old_stat, new_stat),
+    (new_stat ? 1 : 0) - (old_stat ? 1 : 0),
+  };
 }
 
 } // namespace Result
